use crate::constants;
use crate::devices::device::DeviceId;
use crate::energy::energy_actor;
use crate::ota::ota_manager;
use crate::ota::ota_update::OTAPlatform;
use crate::server::custom_error::HttpResult;
use crate::server::WebEnergyActor;
use actix_multipart::form::tempfile::TempFile;
use actix_multipart::form::MultipartForm;
use actix_web::{web, HttpResponse};

pub async fn supported_platforms() -> HttpResult {
    Ok(HttpResponse::Ok().json(vec![OTAPlatform::Wt32Eth01]))
}

#[derive(Debug, MultipartForm)]
pub struct UploadForm {
    #[multipart(rename = "firmware")]
    firmware: Vec<TempFile>,
}

#[derive(serde::Deserialize)]
pub struct UploadPath {
    platform: OTAPlatform,
    version: semver::Version,
}

pub async fn upload_firmware(
    MultipartForm(form): MultipartForm<UploadForm>,
    path: web::Path<UploadPath>,
) -> HttpResult {
    if ota_manager::update_exists(path.platform, &path.version)? {
        return Ok(HttpResponse::Conflict()
            .json("A firmware with the same version has already been uploaded on the platform!"));
    }

    let Some(file) = form.firmware.first() else {
        return Ok(HttpResponse::BadRequest().json("No firmware specified!"));
    };

    if file.size == 0 {
        return Ok(HttpResponse::BadRequest().json("Uploaded file is empty!"));
    }

    if file.size > constants::MAX_FIRMWARE_SIZE {
        return Ok(HttpResponse::BadRequest().json("Uploaded file is too heavy!"));
    }

    let content = std::fs::read(file.file.path())?;

    ota_manager::save_update(path.platform, &path.version, &content)?;

    Ok(HttpResponse::Accepted().body("OTA update successfully saved."))
}

#[derive(serde::Deserialize)]
pub struct SetDesiredDeviceVersion {
    devices: Option<Vec<DeviceId>>,
    platform: Option<OTAPlatform>,
    version: semver::Version,
}

pub async fn set_desired_version(
    actor: WebEnergyActor,
    body: web::Json<SetDesiredDeviceVersion>,
) -> HttpResult {
    if body.devices.is_none() && body.platform.is_none() {
        return Ok(
            HttpResponse::BadRequest().json("Must specify one filter to select target devices!")
        );
    }

    let devices = actor.send(energy_actor::GetDeviceLists).await?;

    for d in devices {
        // Filter per platform
        if let Some(p) = body.platform {
            if d.info.reference != p.to_string() {
                continue;
            }
        }

        // Filter per device
        if let Some(ids) = &body.devices {
            if !ids.contains(&d.id) {
                continue;
            }
        }

        actor
            .send(energy_actor::SetDesiredVersion(
                d.id,
                Some(body.version.clone()),
            ))
            .await??;
    }

    Ok(HttpResponse::Ok().finish())
}
